import random
import logging

from .Discord import Discord
from .... import REDIRECT_URI_NAME
from django.contrib import messages
from django.shortcuts import redirect
from django.views.generic import View
from .models import Discord as DiscordModel
from django.utils.translation import gettext as _
from django.core.exceptions import FieldDoesNotExist
from django.utils.decorators import method_decorator
from django.contrib.auth import get_user_model, login
from django.contrib.auth.decorators import login_required


class Authenticate(View):
    def get(self, request):
        discord = Discord()
        return_url = discord.build_url(request, '__account_discord_authenticated')
        discord_login_url = discord.auth_url(request, return_url)
        return redirect(discord_login_url)


class Authenticated(View):

    def get(self, request):
        discord = Discord()
        redirect_url = discord.build_url(request, '__account_discord_authenticated')
        payload = discord.verify_auth(request, redirect_url)
        self.save_connection(request, payload)
        return redirect(REDIRECT_URI_NAME)

    def save_connection(self, request, payload):
        try:
            discord = Discord()
            identifier, email = discord.validate_payload(payload)

            if not email:
                return False

            data = {
                'identifier': identifier,
                'id_token': payload.get('id_token'),
                'access_token': payload.get('access_token'),
                'expires_in': payload.get('expires_in'),
                'refresh_token': payload.get('refresh_token', None),
                'token_type': payload.get('token_type'),
            }

            user = request.user if not request.user.is_anonymous else None

            if not user:
                is_exist = DiscordModel.objects.filter(email=email).first()
                if not is_exist:
                    payload = {
                        'username': self.clear_username(request, email.split('@')[0].split('+')[0])
                    }

                    # check if email is required
                    if self.is_field_unique(get_user_model(), 'email'):
                        payload.update({"email": email})
                        user = get_user_model().objects.filter(email=email).first()

                    user = get_user_model().objects.create_user(**payload) if not user else user
                    data.update({'user': user})
                else:
                    user = is_exist.user
                DiscordModel.objects.update_or_create(email=email, defaults=data)
                login(request, user)
            else:
                data.update({
                    'email': email,
                })
                is_another_account = DiscordModel.objects.filter(email=email).first()
                if is_another_account:
                    messages.error(self.request, _('account_with_this_connection_already_exist'))
                    return False
                DiscordModel.objects.update_or_create(user=user, defaults=data)
        except Exception as e:
            logging.error(str(e))

    def is_field_unique(self, model, field_name):
        try:
            field = model._meta.get_field(field_name)
            return field.unique
        except FieldDoesNotExist:
            return False

    def clear_username(self, request, username):
        sanitized_username = ''.join(char for char in username if char.isalnum())
        if not sanitized_username:
            return None
        is_exist = get_user_model().objects.filter(username=sanitized_username).first()
        if not is_exist:
            return sanitized_username
        updated_username = f'{sanitized_username}{random.randint(1, 9)}'
        return self.clear_username(request, updated_username)


@method_decorator(decorator=login_required, name='dispatch')
class UnAuthenticate(View):
    def get(self, request):
        if not request.user.email:
            messages.error(self.request, _('add_email_to_unlink_account'))
            return redirect(REDIRECT_URI_NAME)
        discord = Discord()
        discord.revoke_token(request.user)
        return redirect(REDIRECT_URI_NAME)



from django.conf import settings
from .apps import DjAllAuthConfig

config = DjAllAuthConfig
base_template = config.name

__ac_app_name = base_template
__ac_logo = settings.DJ_ALL_AUTH.get('LOGO') if settings.DJ_ALL_AUTH.get('LOGO') else f'{__ac_app_name}/images/logo.webp'
__ac_layout = f'{__ac_app_name}/layouts/base.html'

# css and js list
__ac_css_list = settings.DJ_ALL_AUTH.get('CSS')
__ac_js_list = settings.DJ_ALL_AUTH.get('JS')

REDIRECT_URI_NAME = '/'
if settings.DJ_ALL_AUTH.get('CONNECTIONS'):
    __connection = settings.DJ_ALL_AUTH.get('CONNECTIONS')
    if __connection.get('REDIRECT_URI_NAME') is not None:
        REDIRECT_URI_NAME = __connection.get('REDIRECT_URI_NAME')



from . import __ac_logo, __ac_layout, __ac_app_name, __ac_css_list, __ac_js_list
from django.contrib.sites.shortcuts import get_current_site


def __config(request):
    config = {
        'ac_logo': __ac_logo,  # <- static path
        'ac_layout': __ac_layout,
        'ac_app_name': __ac_app_name,
        'ac_css_list': __ac_css_list,
        'ac_js_list': __ac_js_list,
    }
    return config


def __default_context(request):
    current_site = get_current_site(request)
    domain = f'{request.scheme}://{current_site}'
    site_name = current_site.name
    context = {
        'domain': domain,
        'site_name': site_name
    }
    return context



from django.db import models
from django.utils.translation import gettext as _


class EmailTemplate(models.Model):
    RESET_PASSWORD = 'REP'
    REGISTER = 'REG'
    REQUEST_ACCOUNT_ACTIVATION = 'RAA'
    names = [
        (RESET_PASSWORD, _('reset_password')),
        (REGISTER, _('register')),
        (REQUEST_ACCOUNT_ACTIVATION, _('request_account_activation'))
    ]
    template = models.CharField(max_length=5, choices=names, unique=True)
    content = models.TextField()

    def __str__(self):
        return f"{self.get_template_display()}"



from . import views
from django.urls import path

discord_urlpatterns = [
    path('authenticate', views.Authenticate.as_view(), name='__account_discord_authenticate'),
    path('authenticated', views.Authenticated.as_view(), name='__account_discord_authenticated'),
    path('revoke-authenticate', views.UnAuthenticate.as_view(), name='__account_discord_unauthenticate')
]



from . import base_template
from django.views.generic import TemplateView
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required


@method_decorator(decorator=login_required, name='dispatch')
class Index(TemplateView):
    template_name = f'{base_template}/index.html'



from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserCreationForm

# make default email to unique
get_user_model()._meta.get_field('email')._unique = True


class RegisterForm(UserCreationForm):
    email = forms.EmailField(required=True)

    class Meta:
        model = get_user_model()
        fields = ['username', 'email', 'password1', 'password2']


class AccountActivateForm(forms.Form):
    email = forms.EmailField()



import os
import json
import base64
import hashlib
import logging
import http.client
from django.conf import settings
from urllib.parse import urlencode
from django.shortcuts import reverse
from .models import Discord as DiscordModel
from django.utils.translation import gettext as _
from django.utils.translation import get_language
from django.http.response import HttpResponseForbidden
from django.contrib.sites.shortcuts import get_current_site


class Discord:
    def __init__(self):
        config = settings.DJ_ALL_AUTH.get('CONNECTIONS').get('DISCORD', None) if settings.DJ_ALL_AUTH.get(
            'CONNECTIONS') else settings.DJ_ALL_AUTH.get('CONNECTIONS')

        self.client_id = config.get('CLIENT_ID', None) if config else None
        self.client_secret = config.get('CLIENT_SECRET', None) if config else None

        # set scopes
        self.default_scope = ['openid', 'identify', 'email', 'connections', 'guilds', 'guilds.join']
        self.scope = config.get('SCOPE', self.default_scope) if config else self.default_scope
        self.update_scope()

        self.authentication_url = f'https://discord.com/oauth2/authorize'
        self.oauth2_host = 'discord.com'
        self.oauth2_token = '/api/oauth2/token'
        self.oauth2_revoke = '/oauth2/token/revoke'
        self.api_status = None

    def update_scope(self):
        for scope in self.default_scope:
            if scope not in self.scope:
                self.scope.append(scope)

    def get_domain(self, request):
        current_site = get_current_site(request)
        return f'{request.scheme}://{current_site}'

    def build_url(self, request, name):
        domain = self.get_domain(request)
        redirect_endpoint = reverse(name)
        if get_language():
            redirect_endpoint = redirect_endpoint.replace(f'/{get_language()}', '', 1)
        return f'{domain}{redirect_endpoint}'

    def auth_url(self, request, redirect_url):
        # set session data
        state = hashlib.sha256(os.urandom(1024)).hexdigest()
        request.session['state'] = state

        auth_param = {
            "prompt": 'consent',
            "response_type": 'code',
            "client_id": self.client_id,
            "scope": f'{" ".join(self.scope)}',
            "state": state
        }
        encoded_param = urlencode(auth_param)
        return f'{self.authentication_url}?{encoded_param}&redirect_uri={redirect_url}'

    def verify_auth(self, request, redirect_url):
        response = {}
        try:
            state = request.GET.get('state', None)
            if request.session.get('state') != state:
                logging.error('Invalid access')
                return {}

            payload = {
                'client_id': self.client_id,
                'code': request.GET.get('code', None),
                'client_secret': self.client_secret,
                'redirect_uri': redirect_url,
                'grant_type': 'authorization_code',
            }
            headers = {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
            response = self.call_api(self.oauth2_host, self.oauth2_token, 'POST', payload=payload, headers=headers)
            access_token = response.get('access_token', None)
            refresh_token = response.get('refresh_token', None)
            if response is None or access_token is None or refresh_token is None:
                logging.error('Invalid access')
                return {}
        except Exception as e:
            logging.error(str(e))

        return response

    def validate_payload(self, connection):
        id_token = connection.get('id_token', None)
        if not id_token:
            return False

        header, payload, signature = id_token.split('.')
        payload = self.decode_base64(payload)
        payload = json.loads(payload)
        iss_list = ['https://discord.com', 'discord.com']
        identifier = payload.get('sub')

        if payload.get('iss', None) not in iss_list:
            return False

        if self.client_id not in payload.get('aud'):
            return False

        # TODO add more validation

        # get and return email
        discord_me = '/api/users/@me'
        token_type = connection.get('token_type')
        access_token = connection.get('access_token')
        headers = {
            'Authorization': f'{token_type} {access_token}'
        }

        response = self.call_api(self.oauth2_host, discord_me, 'GET', None, headers)

        return identifier, response.get('email', False)

    def decode_base64(self, payload):
        """Decodes a base64 encoded string with proper padding handling."""
        missing_padding = len(payload) % 4
        if missing_padding != 0:
            payload += '=' * missing_padding

        return base64.b64decode(payload)

    def refresh_token(self, user):
        connection = DiscordModel.objects.filter(user=user).first()
        if connection:
            payload = {
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'refresh_token': connection.refresh_token,
                'grant_type': 'refresh_token',
            }
            headers = {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
            response = self.call_api(self.oauth2_host, self.oauth2_token, 'POST', payload=payload, headers=headers)
            access_token = response.get('access_token', None)
            refresh_token = response.get('refresh_token', None)
            if response is None or access_token is None or refresh_token is None:
                return HttpResponseForbidden(_('invalid_access'))

            connection.access_token = response.get('access_token')
            connection.refresh_token = response.get('refresh_token')
            connection.expires_in = response.get('expires_in')
            connection.token_type = response.get('token_type')
            connection.save()

    def revoke_token(self, user):
        self.refresh_token(user)

        connection = DiscordModel.objects.filter(user=user).first()
        if connection:
            payload = {
                'client_id': self.client_id,
                'token': f'{connection.access_token}',
                'token_type_hint': 'access_token'
            }
            payload = urlencode(payload)
            headers = {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
            self.call_api(self.oauth2_host, self.oauth2_revoke, 'POST', payload, headers)
            connection.delete()

    def call_api(self, host, end_point, method='GET', payload=None, headers=None):
        if headers is None:
            headers = {}

        if payload is not None and not isinstance(payload, str):
            payload = urlencode(payload)
        conn = http.client.HTTPSConnection(host)
        conn.request(method, end_point, payload, headers)
        res = conn.getresponse()
        self.api_status = res.status
        data = res.read()
        try:
            return json.loads(data.decode("utf-8"))
        except Exception as e:
            logging.error(str(e))
            return None



from django.db import models
from django.contrib.auth import get_user_model


class Google(models.Model):
    user = models.OneToOneField(get_user_model(), on_delete=models.CASCADE)
    identifier = models.BigIntegerField(unique=True)
    id_token = models.TextField()
    token_type = models.CharField(max_length=50)
    access_token = models.CharField(max_length=500)
    refresh_token = models.CharField(max_length=500, default=None, null=True)
    expires_in = models.PositiveBigIntegerField()
    email = models.EmailField(unique=True)
    date = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name_plural = 'Google'

    def __str__(self):
        user = str(self.user)
        return f'Google Connection - {user.capitalize()}'



import django.contrib.auth.urls

from . import views
from django.urls import path, include
from . import base_template as template_name
from .services.auth import views as auth_view
from django.contrib.auth import views as d_view

# connection service urls
from .services.connections.steam.urls import steam_urlpatterns
from .services.connections.google.urls import google_urlpatterns
from .services.connections.twitch.urls import twitch_urlpatterns
from .services.connections.discord.urls import discord_urlpatterns

# remove unwanted urls
remove_urls = ['password_change_done', 'password_change']
for name in remove_urls:
    for d in django.contrib.auth.urls.urlpatterns:
        if d.name == name:
            django.contrib.auth.urls.urlpatterns.remove(d)

# After login page
d_view.LoginView.next_page = '/'
d_view.LoginView.template_name = template_name + '/auth/login.html'
d_view.LogoutView.template_name = template_name + '/auth/logout.html'
d_view.PasswordResetView.template_name = template_name + '/auth/password_reset.html'
d_view.PasswordResetDoneView.template_name = template_name + '/auth/password_reset_done.html'
d_view.PasswordResetConfirmView.template_name = template_name + '/auth/password_reset_confirm.html'
d_view.PasswordResetCompleteView.template_name = template_name + '/auth/password_reset_complete.html'

urlpatterns = [
    path('', include("django.contrib.auth.urls")),  # auth urls

    # Auth service
    path('register/', auth_view.Register.as_view(), name='__account_register'),
    path('request-account-activation/', auth_view.ActivateRequest.as_view(), name='__activate_request'),
    path('account-activate/<uidb64>/<token>', auth_view.ActivateAccount.as_view(), name='__activate_account'),

    # connection services
    path('steam/', include(steam_urlpatterns)),
    path('twitch/', include(twitch_urlpatterns)),
    path('google/', include(google_urlpatterns)),
    path('discord/', include(discord_urlpatterns)),
]



from django.db import models
from django.contrib.auth import get_user_model


class Steam(models.Model):
    user = models.OneToOneField(get_user_model(), on_delete=models.CASCADE)
    steam_id = models.CharField(max_length=100, unique=True)
    date = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name_plural = 'Steam'

    def __str__(self):
        user = str(self.user)
        return f'Steam Connection - {user.capitalize()}'



import logging

from .. import is_ajax
from . import auth_template
from ...lib.Mail import Mail
from ... import base_template
from django.urls import reverse
from django.contrib import messages
from django.http import JsonResponse
from django.shortcuts import redirect
from django.contrib.auth import get_user_model
from django.views.generic.edit import FormView
from django.template.loader import get_template
from django.utils.translation import gettext as _
from django.views.generic import CreateView, View
from ...utils.TokenGenerator import TokenGenerator
from ...utils.forms import RegisterForm, AccountActivateForm
from ...context_processors import __default_context as default_context


class Register(CreateView):
    model = get_user_model()
    template_name = auth_template + '/register.html'
    form_class = RegisterForm
    success_url = '/'

    def form_invalid(self, form):
        if is_ajax(self.request):
            return JsonResponse({'status': False, 'errors': form.errors}, status=422)
        else:
            return super().form_invalid(form)

    def form_valid(self, form):
        user = form.save(False)
        user.is_active = False
        user.save()

        # send verification mail
        self.send_email(user)

        messages.success(self.request, _('active_your_account_with_email'))
        if is_ajax(self.request):
            return JsonResponse({'status': True, 'url': reverse('home')}, status=200)
        else:
            return redirect('home')

    def send_email(self, user):
        # Send Account activation mail
        token_generator = TokenGenerator()
        token_info = token_generator.generate_token(user)

        link = reverse("__activate_account",
                       kwargs={"uidb64": token_info.get('uidb64'), "token": token_info.get('token')})
        context = {
            'link': link,
            'user': user
        }
        context.update(default_context(self.request))
        text_content = get_template(f'{base_template}/mail/register/mail.text').render(context)
        args = {
            'subject': _('email_subject_active_request'),
            'to': [user.email],
            'text_content': text_content
        }
        Mail(multipart=True).send_mail(args)


class ActivateAccount(View):
    def get(self, request, uidb64, token):
        try:
            token_generator = TokenGenerator()
            url_hash = token_generator.decode_uid(uidb64)
            user_hash = url_hash.split('|')
            user = get_user_model().objects.filter(id=user_hash[0], username=user_hash[1], email=user_hash[2],
                                                   is_active=False).first()
            if user:
                token_generator = TokenGenerator()
                verify = token_generator.verify_token(user, token)
                if verify:
                    user.is_active = True
                    user.save()
                    messages.info(request, _('account_activated_can_login'))
                else:
                    messages.error(request, _('invalid_token'))
            else:
                messages.error(request, _('no_user_found'))
        except Exception as e:
            logging.error(e)
            messages.error(request, _('something_went_wrong'))
        return redirect('login')


class ActivateRequest(FormView):
    form_class = AccountActivateForm
    template_name = auth_template + '/request_account_activation.html'

    def form_invalid(self, form):
        if is_ajax(self.request):
            return JsonResponse({'status': False, 'errors': form.errors}, status=422)
        else:
            return super().form_invalid(form)

    def form_valid(self, form):
        data = form.cleaned_data
        try:
            user = get_user_model().objects.filter(email=data.get('email'), is_active=False).first()
            if not user:
                form.add_error(None, _('account_with_email_not_exist'))
                return self.form_invalid(form)

            token_generator = TokenGenerator()
            token_info = token_generator.generate_token(user)
            link = reverse("__activate_account",
                           kwargs={"uidb64": token_info.get('uidb64'), "token": token_info.get('token')})
            context = {
                'link': link,
                'user': user
            }
            context.update(default_context(self.request))
            text_content = get_template(f'{base_template}/mail/activate_request/mail.text').render(context)
            args = {
                'subject': _('email_subject_active_request'),
                'to': [user.email],
                'text_content': text_content,
            }
            Mail(multipart=True).send_mail(args)

            messages.info(self.request, _('active_your_account_with_email'))
            if is_ajax(self.request):
                return JsonResponse({'status': True, 'url': reverse('login')}, status=200)
            else:
                return redirect('login')
        except Exception as e:
            logging.error(e)
            form.add_error(None, _('something_went_wrong'))
            return self.form_invalid(form)



from django.utils.encoding import force_str
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.utils.http import urlsafe_base64_decode
from django.contrib.auth.tokens import PasswordResetTokenGenerator


class TokenGenerator:
    def generate_token(self, user):
        user_hash = f'{user.pk}|{user.username}|{user.email}|{user.is_active}'
        uid_64 = urlsafe_base64_encode(force_bytes(user_hash))

        account_activation_token = PasswordResetTokenGenerator()
        token = PasswordResetTokenGenerator.make_token(self=account_activation_token, user=user)

        token_info = {
            'uidb64': uid_64,
            'token': token
        }
        return token_info

    def verify_token(self, user, token):
        account_activation_token = PasswordResetTokenGenerator()
        return PasswordResetTokenGenerator.check_token(self=account_activation_token, user=user, token=token)

    def decode_uid(self, uidb64):
        return force_str(urlsafe_base64_decode(uidb64))



import os
import logging

from django.core.mail import EmailMessage, EmailMultiAlternatives


class Mail:

    def __init__(self, multipart=False):
        self.multipart = multipart
        self.mailer = EmailMessage
        if self.multipart:
            self.mailer = EmailMultiAlternatives

    def send_mail(self, args=None):
        try:
            if args.get('to'):
                text_content = args.get('text_content') if args.get('text_content') else None
                html_content = args.get('html_content') if args.get('html_content') else None

                mail = self.mailer(
                    subject=args.get('subject') if args.get('subject') else "",
                    body=text_content,
                    to=args.get('to'),
                    cc=args.get('cc') if args.get('cc') else None,
                    bcc=args.get('bcc') if args.get('bcc') else None,
                    attachments=args.get('attachments') if args.get('attachments') else None,
                    reply_to=[os.getenv('EMAIL_NO_REPLY')],
                )
                if html_content and self.multipart:
                    mail.attach_alternative(html_content, "text/html")
                mail.send()
        except Exception as e:
            logging.exception(str(e))



import os
import uuid
import json
import base64
import hashlib
import logging
import http.client
from django.conf import settings
from urllib.parse import urlencode
from django.shortcuts import reverse
from .models import Google as GoogleModel
from django.utils.translation import gettext as _
from django.utils.translation import get_language
from django.http.response import HttpResponseForbidden
from django.contrib.sites.shortcuts import get_current_site


class Google:

    def __init__(self):
        config = settings.DJ_ALL_AUTH.get('CONNECTIONS').get('GOOGLE', None) if settings.DJ_ALL_AUTH.get(
            'CONNECTIONS') else None

        self.client_id = config.get('CLIENT_ID', None) if config else None
        self.client_secret = config.get('CLIENT_SECRET', None) if config else None

        # set scopes
        self.default_scope = ['openid', 'email']
        self.scope = config.get('SCOPE', self.default_scope) if config else self.default_scope
        self.update_scope()

        self.authentication_url = f'https://accounts.google.com/o/oauth2/v2/auth'
        self.oauth2_host = 'oauth2.googleapis.com'
        self.oauth2_token = '/token'
        self.oauth2_revoke = '/revoke'
        self.api_status = None

    def update_scope(self):
        for scope in self.default_scope:
            if scope not in self.scope:
                self.scope.append(scope)

    def get_domain(self, request):
        current_site = get_current_site(request)
        return f'{request.scheme}://{current_site}'

    def build_url(self, request, name):
        domain = self.get_domain(request)
        redirect_endpoint = reverse(name)
        if get_language():
            redirect_endpoint = redirect_endpoint.replace(f'/{get_language()}', '', 1)
        return f'{domain}{redirect_endpoint}'

    def auth_url(self, request, redirect_url):
        # set session data
        state = hashlib.sha256(os.urandom(1024)).hexdigest()
        nonce = str(uuid.uuid4())
        request.session['state'] = state

        auth_param = {
            "prompt": 'consent',
            "response_type": 'code',
            "client_id": self.client_id,
            "scope": f'{" ".join(self.scope)}',
            "state": state,
            "login_hint": request.user.email if request.user and not request.user.is_anonymous else '',
            "nonce": nonce,
            "hd": self.get_domain(request),
            "access_type": 'offline'
        }
        encoded_param = urlencode(auth_param)
        return f'{self.authentication_url}?{encoded_param}&redirect_uri={redirect_url}'

    def verify_auth(self, request, redirect_url):
        response = {}
        try:
            state = request.GET.get('state', None)
            if request.session.get('state') != state:
                logging.error('Invalid access')
                return {}

            payload = {
                'code': request.GET.get('code', None),
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'redirect_uri': redirect_url,
                'grant_type': 'authorization_code',
            }
            response = self.call_api(self.oauth2_host, self.oauth2_token, 'POST', payload=payload)
            access_token = response.get('access_token', None)
            refresh_token = response.get('refresh_token', None)
            if response is None or access_token is None or refresh_token is None:
                logging.error('Invalid access')
                return {}
        except Exception as e:
            logging.error(str(e))

        return response

    def validate_payload(self, connection):
        id_token = connection.get('id_token', None)
        if not id_token:
            return False

        header, payload, signature = id_token.split('.')
        payload = self.decode_base64(payload)
        payload = json.loads(payload)
        iss_list = ['https://accounts.google.com', 'accounts.google.com']
        identifier = payload.get('sub')

        if payload.get('iss', None) not in iss_list:
            return False

        if payload.get('aud') != self.client_id:
            return False

        # TODO add more validation

        return identifier, payload.get('email', False)

    def decode_base64(self, payload):
        """Decodes a base64 encoded string with proper padding handling."""
        missing_padding = len(payload) % 4
        if missing_padding != 0:
            payload += '=' * missing_padding

        return base64.b64decode(payload)

    def refresh_token(self, user):
        connection = GoogleModel.objects.filter(user=user).first()
        if connection:
            payload = {
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'refresh_token': connection.refresh_token,
                'grant_type': 'refresh_token',
            }
            response = self.call_api(self.oauth2_host, self.oauth2_token, 'POST', payload=payload)
            access_token = response.get('access_token', None)
            if response is None or access_token is None:
                return HttpResponseForbidden(_('invalid_access'))

            connection.access_token = response.get('access_token')
            connection.expires_in = response.get('expires_in')
            connection.token_type = response.get('token_type')
            connection.save()

    def revoke_token(self, user):
        connection = GoogleModel.objects.filter(user=user).first()
        if connection:
            payload = {
                'token': connection.access_token
            }
            self.call_api(self.oauth2_host, self.oauth2_revoke, 'POST', payload=payload)
            connection.delete()

    def call_api(self, host, end_point, method='GET', payload=None, headers=None):
        if headers is None:
            headers = {}

        if payload is not None:
            payload = json.dumps(payload)
        conn = http.client.HTTPSConnection(host)
        conn.request(method, end_point, payload, headers)
        res = conn.getresponse()
        self.api_status = res.status
        data = res.read()
        try:
            return json.loads(data.decode("utf-8"))
        except Exception as e:
            logging.error(str(e))
            return None



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import gettext as _
from .models import Google, Steam, Twitch, Discord, EmailTemplate


class CustomUserModel(UserAdmin):
    def get_fieldsets(self, request, obj=None):
        fieldsets = super(CustomUserModel, self).get_fieldsets(request, obj)
        if not request.user.is_superuser and obj:
            fieldsets = (
                (None, {"fields": ("username", "password")}),
                (_("Personal info"), {"fields": ("first_name", "last_name", "email")}),
                (_("Activation"), {"fields": ("is_active",), },),
                (_("Important dates"), {"fields": ("last_login", "date_joined")}),
            )
        return fieldsets

    # list only staff user
    def get_queryset(self, request):
        if request.user.is_superuser:
            return get_user_model().objects.filter()
        else:
            return get_user_model().objects.filter(is_superuser=False)


class EmailTemplateAdmin(admin.ModelAdmin):
    search_fields = ('template',)
    list_filter = ('template',)

    class Media:
        js = ['tinymce/tinymce.min.js', 'js/admin/django-admin-email-template.js']


class DefaultPermission(admin.ModelAdmin):
    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class GoogleAdmin(DefaultPermission):
    search_fields = ('user__username', 'token_type', 'access_token', 'expires_in', 'date')
    list_filter = ('token_type',)
    list_display = ('user', 'token_type', 'access_token', 'expires_in', 'date')


class SteamAdmin(DefaultPermission):
    search_fields = ('user__username', 'steam_id', 'date')
    list_display = ('user', 'steam_id', 'date')


class TwitchAdmin(DefaultPermission):
    search_fields = ('user__username', 'token_type', 'access_token', 'expires_in', 'date')
    list_filter = ('token_type',)
    list_display = ('user', 'token_type', 'access_token', 'expires_in', 'date')


class DiscordAdmin(DefaultPermission):
    search_fields = ('user__username', 'identifier', 'expires_in', 'date')
    list_filter = ('token_type',)
    list_display = ('user', 'identifier', 'expires_in', 'date')


admin.site.unregister(get_user_model())
admin.site.register(get_user_model(), CustomUserModel)
admin.site.register(EmailTemplate, EmailTemplateAdmin)
admin.site.register(Google, GoogleAdmin)
admin.site.register(Steam, SteamAdmin)
admin.site.register(Twitch, TwitchAdmin)
admin.site.register(Discord, DiscordAdmin)
